;*********************************** Int_INT_02.asm *************************************
;
;
; Cada vez que presiona el pulsador conectado al pin RB0/INT se incrementa un contador
; que es visualizado en el mdulo LCD. La lectura del pulsador se har mediante interrupciones.
;
; Se supone que el pulsador conectado al pin RB0 en la EasyPIC es activo a nivel alto, es decir,
; cuando est pulsado RB0=5V.
;
; ZONA DE DATOS **********************************************************************

	__CONFIG   _CP_OFF &  _WDT_OFF & _PWRTE_ON & _HS_OSC
	LIST	   P=16F877A
	INCLUDE  <P16F877A.INC>

	CBLOCK  0x20
	Contador				; El contador a visualizar.
	ENDC

#DEFINE  Pulsador 	PORTB,0		; Lnea donde se conecta el pulsador.

; ZONA DE CDIGOS ********************************************************************

	ORG 	0
	goto	Inicio
	ORG	4			; Vector de interrupcin.
	goto	ServicioInterrupcion
Inicio
	call	LCD_Inicializa
	bsf	STATUS,RP0		; Acceso al Banco 1.
	bsf	Pulsador			; La lnea RB0/INT se configura como entrada.
	bsf	OPTION_REG,INTEDG	; Interrupcin INT se activa por flanco de SUBIDA.
	bcf	STATUS,RP0		; Acceso al Banco 0.
	clrf	Contador			; Inicializa el contador y lo visualiza.
	call	VisualizaContador
	movlw	b'10010000'		; Habilita la interrupcin INT y la general.
	movwf	INTCON

; La seccin "Principal" es de mantenimiento. Slo espera las interrupciones
; en modo de bajo consumo.

Principal
	sleep				; Pasa a modo de bajo consumo o reposo.
	goto	Principal

; Subrutina "ServicioInterrupcion" ------------------------------------------------------
;
; Subrutina de servicio a la interrupcin. Incrementa un contador y lo visualiza.
;
ServicioInterrupcion
	call	Retardo_20ms		; Espera a que se estabilice el nivel de tensin.
	btfss	Pulsador			; Comprueba si es un rebote.
	goto	FinInterrupcion		; Era un rebote y por tanto sale.
	incf	Contador,F		; Incrementa el contador y lo visualiza.
VisualizaContador
	call	LCD_Linea1
	movf	Contador,W
	call	BIN_a_BCD		; Se debe visualizar en BCD.
	call	LCD_Byte
FinInterrupcion
	bcf	INTCON,INTF		; Limpia flag de reconocimiento (INTF).
	retfie				; Retorna y rehabilita las interrupciones (GIE=1).

	INCLUDE   <RETARDOS.INC>
	INCLUDE   <BIN_BCD.INC>
	INCLUDE   <LCD_EASY.INC>
	END


