;*********************************** Int_INT_07.asm *************************************
;
;
; Comprueba el funcionamiento de la interrupcin por activacin del pin RB0/INT y analiza
; cmo deben guardarse los datos que se corrompen durante el proceso de la llamada a subrutina.
;
; Cada vez que presione el pulsador conectado al pin RB0/INT conmutar el estado de los LEDs
; conectados a PORTC. Al mismo tiempo en el mdulo LCD se visualizar un mensaje
; desplazndose por pantalla.
;
; ZONA DE DATOS **********************************************************************

	__CONFIG   _CP_OFF &  _WDT_OFF & _PWRTE_ON & _HS_OSC

	LIST	   P=16F877A
	INCLUDE  <P16F877A.INC>

	CBLOCK  0x20
	ENDC

#DEFINE  Pulsador  PORTB,0		; Lnea donde se conecta el pulsador.
#DEFINE  LUCES	   PORTC		; LEDS EN EL PUERTO PORTC
#DEFINE  LED	   PORTC,0

; ZONA DE CDIGOS ********************************************************************

	ORG 	0
	goto	Inicio
	ORG	4			; Vector de interrupcin.
	goto	ServicioInterrupcion
Inicio
	call	LCD_Inicializa
	bsf	STATUS,RP0		; Acceso al Banco 1.
	bsf	Pulsador		; La lnea RB0/INT se configura como entrada.
	clrf LUCES			; Se configura como salida.
	bsf	OPTION_REG,INTEDG	; Interrupcin INT se activa por flanco de SUBIDA
	bcf	STATUS,RP0		; Acceso al Banco 0.
	movlw	b'10010000'		; Habilita la interrupcin INT y la general.
	movwf	INTCON
Principal
	movlw	MensajeLargo		; Visualiza el mensaje desplazndose por la
	call	LCD_MensajeMovimiento	; pantalla.
	goto	Principal

; Subrutina "ServicioInterrupcion" ------------------------------------------------------
;
; Subrutina de atencin a la interrupcin. Conmuta el estado del LED.

; Como esta subrutina altera los valores del registro de trabajo W, del STATUS, y de los
; registros R_ContA y R_ContB utilizados en los retardos, habr que preservar su valor
; previo y despus restaurarlo al final.

	CBLOCK	
	Guarda_W
	Guarda_STATUS
	Guarda_R_ContA
	Guarda_R_ContB
	ENDC

ServicioInterrupcion
	movwf	Guarda_W		; Guarda W y STATUS.
	swapf	STATUS,W		; Ya que "movf STATUS,W" corrompe el bit Z.
	movwf	Guarda_STATUS
	bcf	STATUS,RP0		; Para asegurarsede que trabaja con el Banco 0.
	movf	R_ContA,W		; Guarda los registros utilizados en esta 
	movwf	Guarda_R_ContA		; subrutina y tambin en la principal.
	movf	R_ContB,W
	movwf	Guarda_R_ContB
;
	call	Retardo_20ms
	btfss	Pulsador		; Comprueba si es un rebote.
	goto	FinInterrupcion		; Era un rebote y por tanto sale.
	btfsc	LED			; Testea el ltimo estado de las luces (en concreto el  LED de RC0).
 	goto	EstabaEncendido
EstabaApagado
	movlw 	0xFF
	movwf 	LUCES			; Estaba apagado y lo enciende.
	goto	FinInterrupcion
EstabaEncendido
	clrf	LUCES			; Estaba encendido y lo apaga.
FinInterrupcion
	swapf	Guarda_STATUS,W		; Restaura el STATUS.
	movwf	STATUS
	swapf	Guarda_W,F		; Restaura W como estaba antes de producirse
	swapf	Guarda_W,W		; interrupcin.
	movf	Guarda_R_ContA,W	; Restaura los registros utilizados en esta 
	movwf	R_ContA			; subrutina y tambin en la principal.
	movf	Guarda_R_ContB,W
	movwf	R_ContB
	bcf	INTCON,INTF		; Limpia flag de reconocimiento de la interrupcin.
	retfie				; Retorna y rehabilita las interrupciones.

; "Mensajes" ----------------------------------------------------------------------------

Mensajes
	addwf	PCL,F		
MensajeLargo
	DT " 		    "
	DT "Me gusta desarrollar proyectos con PICs."
	DT "                ", 0x00

	INCLUDE   <LCD_MENS.INC>
	INCLUDE   <LCD_EASY.INC>
	INCLUDE   <RETARDOS.INC>
	END

