;************************************ Int_RBI_01.asm ************************************
;

; Este programa comprueba el funcionamiento de la interrupcin por cambio de estado de una
; lnea de la parte alta del Puerto B, por ejemplo la RB6. Cada vez que presiona el pulsador
; conectado al pin RB6 se incrementar un contador que se visualiza en el mdulo LCD.
;
; Se supone que el pulsador conectado al pin RB6 en la EasyPIC es activo a nivel alto, es decir,
; cuando est pulsado RB6=5V.

; ZONA DE DATOS **********************************************************************

	__CONFIG   _CP_OFF &  _WDT_OFF & _PWRTE_ON & _HS_OSC
	LIST	   P=16F877A
	INCLUDE  <P16F877A.INC>

	CBLOCK   0x20
	Contador
	ENDC

#DEFINE   Pulsador	PORTB,6		; Lnea donde se conecta el pulsador.

; ZONA DE CDIGOS ********************************************************************

	ORG 	0
	goto	Inicio
	ORG	4
	goto	ServicioInterrupcion
Inicio
	call	LCD_Inicializa
	bsf	STATUS,RP0
	bsf	Pulsador		; La lnea se configura como entrada.
;	bcf	OPTION_REG,NOT_RBPU	; Activa las resistencias de Pull-up del Puerto B.
	bcf	STATUS,RP0
	clrf	Contador		; Inicializa el contador y lo visualiza.
	call	VisualizaContador
	movlw	b'10001000'		; Habilita la interrupcin RBI y la general.
	movwf	INTCON
Principal
	sleep				; Pasa a modo de bajo consumo y espera las
	goto	Principal		; interrupciones.

; Subrutina "ServicioInterrupcion" ------------------------------------------------------
;
; Subrutina de atencin a la interrupcin. Incrementa el contador y lo visualiza.
;
ServicioInterrupcion
	call	Retardo_20ms		; Para salvaguardar de los rebotes.
	btfss	Pulsador		; Comprueba si es un rebote.
	goto	FinInterrupcion		; Era un rebote y por tanto sale.
	incf	Contador,F		; Incrementa el contador y lo visualiza.
VisualizaContador
	call	LCD_Linea1		; Se pone al principio de la lnea 1.
	movf	Contador,W
	call	BIN_a_BCD		; Se debe visualizar en BCD.
	call	LCD_Byte
EsperaDejePulsar
	btfsc	Pulsador		; Para que no interrumpa tambin en el flanco
	goto	EsperaDejePulsar	; de bajada cuando suelte el pulsador.
FinInterrupcion
	bcf	INTCON,RBIF		; Limpia flag de reconocimiento RBIF.
	retfie

	INCLUDE   <RETARDOS.INC>
	INCLUDE   <BIN_BCD.INC>
	INCLUDE   <LCD_EASY.INC>
	END

