;************************************ Int_RBI_05.asm **********************************
;
; A las lneas RB7 y RB6 se conectan dos pulsadores que producen una interrupcin cada vez
; que se pulsan. En el mdulo LCD se visualizar el nombre del pulsador activado: "RB7" o "RB6".
;
; ZONA DE DATOS **********************************************************************

	__CONFIG   _CP_OFF &  _WDT_OFF & _PWRTE_ON & _HS_OSC
	LIST	   P=16F877A
	INCLUDE  <P16F877A.INC>

	CBLOCK  0x20
	Contador
	ENDC

#DEFINE  EntradaRB7	PORTB,7
#DEFINE  EntradaRB6	PORTB,6

; ZONA DE CDIGOS ********************************************************************

	ORG 	0
	goto	Inicio
	ORG	4			; Vector de interrupcin.
	goto	ServicioInterrupcion
Inicio
	call	LCD_Inicializa
	movlw	MensajeInicial
	call	LCD_Mensaje		; Visualiza el mensaje inicial.
	bsf	STATUS,RP0		; Acceso al Banco 1.
	bsf	EntradaRB7		; Las lneas se configuran como entrada.
	bsf	EntradaRB6
	bcf	STATUS,RP0		; Acceso al Banco 0.
	movlw	b'10001000'		; Activa interrupcin  por cambio en las
	movwf	INTCON			; lneas del Puerto B (RBIE) y la general (GIE).
Principal
	sleep				; Pasa a modo bajo consumo esperando las
	goto 	Principal		; interrupciones.

; Subrutina "ServicioInterrupcion" ------------------------------------------------------
;
; Subrutina de atencin a la interrupcin. Detecta qu ha producido la interrupcin y
; ejecuta la subrutina correspondiente.

ServicioInterrupcion
	call	Retardo_20ms		; Espera se estabilicen niveles.	
	btfsc	EntradaRB7		; Est presionado el pulsador RB7?
	call	VisualizaRB7
	btfsc	EntradaRB6		; Est presionado el pulsador RB6?
	call	VisualizaRB6
	bcf	INTCON,RBIF
	retfie				; Retorna y rehabilita las interrupciones, GIE=1.

; Subrutinas "VisualizaRB7" y "VisualizaRB6" --------------------------------------------

VisualizaRB7
	call	LCD_Borra
	movlw	MensajeRB7		; Visualiza el mensaje para RB7.
	call	LCD_Mensaje
	return

VisualizaRB6
	call	LCD_Borra
	movlw	MensajeRB6		; Visualiza el mensaje para RB6.
	call	LCD_Mensaje
	return

; "Mensajes" ----------------------------------------------------------------------------
;
Mensajes
	addwf	PCL,F
MensajeInicial
	DT "Editorial  Ra-Ma", 0x00
MensajeRB7
	DT "  Pulsador RB7", 0x00
MensajeRB6
	DT "   Ahora RB6  ", 0x00

	INCLUDE   <LCD_EASY.INC>
	INCLUDE   <LCD_MENS.INC>
	INCLUDE   <RETARDOS.INC>
	END

